--------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    11:56:55 10/25/2011 
-- Design Name:    I2C slave register HDL
-- Module Name:    i2c_regs - Behavioral 
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions:  
-- Description:    This is the HDL that holds all the register of the I2C slave
--                 core
--
-- The registers should be programmed in the following order:
--   1.- CTR0 Sets up the control register: enable, start, i2c address
--   2.- CTR1 Sets up the threshold for detection signals in RX and TX buffers
--
-- Revision: 
-- Revision 0.01 - File Created
--          1.0  - Stable version
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity i2c_regs is
   port (
      pf_wb_addr_i     : in  STD_LOGIC;
      rd_done_i        : in  STD_LOGIC;
      wr_done_i        : in  STD_LOGIC;

      wb_rst_i         : in   STD_LOGIC;
      wb_clk           : in   STD_LOGIC;

      wb_master_we_o   : out STD_LOGIC;
      wb_master_stb_o  : out STD_LOGIC;
      wb_master_cyc_o  : out STD_LOGIC;
      wb_master_sel_o  : out STD_LOGIC_VECTOR (3  downto 0);
      wb_master_data_i : in  STD_LOGIC_VECTOR (31 downto 0);
      wb_master_data_o : out STD_LOGIC_VECTOR (31 downto 0);
      wb_master_addr_o : out STD_LOGIC_VECTOR (15  downto 0);
      wb_master_ack_i  : in  STD_LOGIC;
      wb_master_rty_i  : in  STD_LOGIC;
      wb_master_err_i  : in  STD_LOGIC;

      -- These are the registers offers to others modules of the FPGA
      wb_slave_we_i   : in   STD_LOGIC;
      wb_slave_stb_i  : in   STD_LOGIC;
      wb_slave_cyc_i  : in   STD_LOGIC;
      wb_slave_sel_i  : in   STD_LOGIC_VECTOR (3  downto 0);
      wb_slave_data_i : in   STD_LOGIC_VECTOR (31 downto 0);
      wb_slave_data_o : out  STD_LOGIC_VECTOR (31 downto 0);
      wb_slave_addr_i : in   STD_LOGIC_VECTOR (3  downto 0);
      wb_slave_ack_o  : out  STD_LOGIC;
      wb_slave_rty_o  : out  STD_LOGIC;
      wb_slave_err_o  : out  STD_LOGIC;

      i2c_addr       : in   STD_LOGIC_VECTOR (6  downto 0);
      -- These are the registers that are offered to the i2c slave core
      I2C_MASK_o    : out  STD_LOGIC_VECTOR (6 downto 0);
      CTR0_o    : out  STD_LOGIC_VECTOR (15 downto 0);
      CTR1_o    : out  STD_LOGIC_VECTOR (15 downto 0);
      STA_i     : in   STD_LOGIC_VECTOR (15 downto 0);
      DRX0_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX1_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX2_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX3_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX4_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX5_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DTX0_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX1_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX2_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX3_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      -- Controls for FIFO_dispatcher
      load_TX     : out  STD_LOGIC
   );
end i2c_regs;

architecture Behavioral of i2c_regs is

   -------------------------------------
   -- STATUS register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  0         EN             General ENable
   --  1         RST            General ReSeT
   --  2         RD_WRN_INST    Reserved
   --  3         A_RX          
   --  4         A_TX          
   --  8-5       x              Reserved
   --  15-9      i2c_sla_fsm    i2c fsm
   -------------------------------------
  constant  STA_addr    : STD_LOGIC_VECTOR(3 downto 0) := X"0";

   -------------------------------------
   -- I2C MASK register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  6-0       MASK           I2C MASK
   --  15-7      x              Reserved
   -------------------------------------

   constant I2C_MASK_addr     : STD_LOGIC_VECTOR(3 downto 0) := X"1";
   -------------------------------------
   -- CTR0 register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Write-read
   ------------------------------------
   -- BIT       NAME           Description
   --  0         EN             general ENable
   --  1         RST            ReSeT
   --  2         PEN            Prescaler ENable
   --  5-3       x              Reserved
   --  7-6       BIA            Bytes Indirect Addressing
   --  14-8      A[6:0]         I2c address
   --  15        x              Reserved
   -------------------------------------
   constant  CTR0_addr   : STD_LOGIC_VECTOR(3 downto 0) := X"2";

   -------------------------------------
   -- CTR1 register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  6-0       BRD            Bytes to be ReaD from FPGA
   --  13-7      BWR            Bytes to be WRitten to FPGA
   --  15-14     WAL            Wishbone Addressing byte Lenght
   -------------------------------------
   constant  CTR1_addr   : STD_LOGIC_VECTOR(3 downto 0) := X"3";

   -------------------------------------
   -- DRXA register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  7-0       DRX0           Data RX register 0      
   --  15-8      DRX1           Data RX register 1
   --  23-16     DRX2           Data RX register 2
   --  31-24     DRX3           Data RX register 3
   -------------------------------------
   constant  DRXA_addr   : STD_LOGIC_VECTOR(3 downto 0) := X"4";
   
   -------------------------------------
   -- DRXB register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  7-0       DRX5           Data RX register 5 
   --  15-8      DRX6           Data RX register 6 
   --  31-16     x              Reserved
   -------------------------------------
   constant  DRXB_addr   : STD_LOGIC_VECTOR(3 downto 0) := X"5";

   -------------------------------------
   -- DTX register
   -------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only
   ------------------------------------
   -- BIT       NAME           Description
   --  7-0       DTX0           Data TX register 0 
   --  15-8      DTX1           Data TX register 1 
   --  23-16     DTX2           Data TX register 2 
   --  31-24     DTX3           Data TX register 3 
   -------------------------------------
   constant  DTX_addr    : STD_LOGIC_VECTOR(3 downto 0) := X"6";

   type WB_BASIC_fsm is (S0_IDLE, S1A_WB_RD_RQT, S1B_WB_WR_RQT,
                         S2_WB_NOOP, S3_WB_ACK, Q0_OK, Q1_ERROR);

signal i2c_master_WB_BASIC_fsm : WB_BASIC_fsm;

signal wb_slave_ack_s : STD_LOGIC;
signal wb_slave_rty_s : STD_LOGIC;
signal wb_slave_err_s : STD_LOGIC;

signal STA_s       : STD_LOGIC_VECTOR  (15 downto 0);
signal I2C_MASK_s  : STD_LOGIC_VECTOR  (15  downto 0);
signal CTR0_s      : STD_LOGIC_VECTOR  (14  downto 0);
signal CTR1_s      : STD_LOGIC_VECTOR  (15 downto 0);

signal DTX0_s : STD_LOGIC_VECTOR(7 downto 0);
signal DTX1_s : STD_LOGIC_VECTOR(7 downto 0);
signal DTX2_s : STD_LOGIC_VECTOR(7 downto 0);
signal DTX3_s : STD_LOGIC_VECTOR(7 downto 0);

signal load_TX_s     : STD_LOGIC;

begin

I2C_MASK_o              <= I2C_MASK_s(6 downto 0);
CTR0_o(15)          <= '0';
CTR0_o(14 downto 0) <= CTR0_s(14 downto 0);
CTR1_o              <= CTR1_s;
DTX0_o              <= DTX0_s;
DTX1_o              <= DTX1_s;
DTX2_o              <= DTX2_s;
DTX3_o              <= DTX3_s;
wb_slave_ack_o            <= wb_slave_ack_s;
wb_slave_rty_o            <= wb_slave_rty_s;
wb_slave_err_o            <= wb_slave_err_s;
load_TX             <= load_TX_s;

wbmaster_reg_proc: process(wb_clk)
begin
   if rising_edge(wb_clk) then
      if wb_rst_i = '1' then
         i2c_master_WB_BASIC_fsm  <=  S0_IDLE;
      else

         case i2c_master_WB_BASIC_fsm is
            when S0_IDLE   =>
               if pf_wb_addr_i = '1' then
                  i2c_master_WB_BASIC_fsm <= S1A_WB_RD_RQT;
               elsif rd_done_i = '1' then 
                  i2c_master_WB_BASIC_fsm <= S1B_WB_WR_RQT;
               else
                  i2c_master_WB_BASIC_fsm <= S0_IDLE;
               end if;
            when S1A_WB_RD_RQT  =>
               i2c_master_WB_BASIC_fsm <= S2_WB_NOOP;
            when S1B_WB_WR_RQT  =>
               i2c_master_WB_BASIC_fsm <= S2_WB_NOOP;
            when S2_WB_NOOP =>
               i2c_master_WB_BASIC_fsm <= S3_WB_ACK;
            when S3_WB_ACK  =>
               i2c_master_WB_BASIC_fsm <= Q0_OK;
            when Q0_OK      =>
               i2c_master_WB_BASIC_fsm <= S0_IDLE;
            when others     =>
               i2c_master_WB_BASIC_fsm <= S0_IDLE;
         end case;

      end if;
   else
   end if;
end process wbmaster_reg_proc;

wbmaster_comb_proc: process(i2c_master_WB_BASIC_fsm)
begin
   case i2c_master_WB_BASIC_fsm is
      when S0_IDLE        =>
         wb_master_cyc_o   <= '0'; 
         wb_master_stb_o   <= '0'; 
         wb_master_we_o    <= '0'; 
         wb_master_sel_o   <= (others => '0');
         wb_master_data_o  <= (others => '0'); 
         wb_master_addr_o  <= (others => '0'); 
      -- This case is accessed when prefetching a wishbone
      -- address, in that case, the first two bytes of the
      -- DRXi registers hold the wishbone address.
      when S1A_WB_RD_RQT  =>
         wb_master_cyc_o       <= '1';
         wb_master_stb_o       <= '1';
         wb_master_we_o        <= '0';
         wb_master_sel_o       <= X"F";
         wb_master_data_o      <= (others => '0'); 
         wb_master_addr_o      <= DRX1_i & DRX2_i;
      when S1B_WB_WR_RQT  =>
         wb_master_cyc_o       <= '1';
         wb_master_stb_o       <= '1';
         wb_master_we_o        <= '1';
         wb_master_sel_o       <= X"F";
         wb_master_data_o      <= DRX3_i & DRX2_i & DRX1_i & DRX0_i;
         wb_master_addr_o      <= DRX5_i & DRX4_i;
      when S2_WB_NOOP     =>
         -- stb_o works like in wb pipelined mode
         wb_master_stb_o       <= '0';
         wb_master_cyc_o       <= '0';
         wb_master_we_o        <= '0';
         wb_master_sel_o       <= X"0";
         wb_master_data_o      <= (others => '0'); 
         wb_master_addr_o      <= (others => '0'); 
      when S3_WB_ACK      =>
         if wb_master_ack_i = '1' then
            wb_master_stb_o       <= '0';
            wb_master_cyc_o       <= '0';
            wb_master_we_o        <= '0';
            wb_master_sel_o       <= X"0";
            wb_master_data_o      <= (others => '0'); 
            wb_master_addr_o      <= (others => '0'); 
--         elsif wb_master_rty_i = '1' then
         else
            wb_master_stb_o       <= '0';
            wb_master_cyc_o       <= '0';
            wb_master_we_o        <= '0';
            wb_master_sel_o       <= X"0";
            wb_master_data_o      <= (others => '0'); 
            wb_master_addr_o      <= (others => '0'); 
         end if;
      when Q0_OK          =>
            wb_master_stb_o       <= '0';
            wb_master_cyc_o       <= '0';
            wb_master_we_o        <= '0';
            wb_master_sel_o       <= X"0";
            wb_master_data_o      <= (others => '0'); 
            wb_master_addr_o      <= (others => '0'); 
      when others         =>
            -- If we enter in a strange state we release the wb bus to
            -- avoid interlocks
            wb_master_stb_o       <= '0';
            wb_master_cyc_o       <= '0';
            wb_master_we_o        <= '0';
            wb_master_sel_o       <= X"0";
            wb_master_data_o      <= (others => '0'); 
            wb_master_addr_o      <= (others => '0'); 
   end case;
end process;

wbslave_proc: process (wb_clk)
   begin
      if rising_edge(wb_clk) then
         if wb_rst_i = '1' then
            wb_slave_data_o    <= (others => '0');
            wb_slave_ack_s     <= '0';
            wb_slave_rty_s     <= '0';
            wb_slave_err_s     <= '0';
            -- This is the default maybe it is good to place the
            -- autoconfiguration here.
            I2C_MASK_s(6  downto 0)         <= "0011111";
            I2C_MASK_s(15 downto 7)        <= (others => '0');
            CTR0_s(0)           <= '1';
            CTR0_s(5  downto 1)  <= (others => '0');
            CTR0_s(7  downto 6)  <= "10";
            CTR0_s(14 downto 8)  <= i2c_addr AND "0011111";
            CTR1_s(6  downto 0)  <= "000" & X"4";
            CTR1_s(13 downto 7)  <= "000" & X"4";
            CTR1_s(15 downto 14) <= "10";
            DTX0_s  <= (others => '0');
            DTX1_s  <= (others => '0');
            DTX2_s  <= (others => '0');
            DTX3_s  <= (others => '0');
            load_TX_s     <= '0';
         elsif (wb_slave_stb_i = '1' and wb_slave_cyc_i = '1') then
            if (wb_slave_ack_s or wb_slave_rty_s or wb_slave_err_s) = '1' then
               wb_slave_ack_s   <= '0';
               wb_slave_rty_s   <= '0';
               wb_slave_err_s   <= '0';
            else
            case wb_slave_we_i is 
            when '1' =>
               case wb_slave_addr_i is        
              when I2C_MASK_addr => 
                  I2C_MASK_s(6  downto 0)  <= wb_slave_data_i(6  downto 0); 
                  I2C_MASK_s(15 downto 7)  <= (others => '0'); 
                  wb_slave_ack_s   <= '1';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '0';
              when CTR0_addr => 
                  CTR0_s           <= wb_slave_data_i(14  downto 0); 
                  wb_slave_ack_s   <= '1';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '0';
               when CTR1_addr => 
                  CTR1_s           <= wb_slave_data_i(15  downto 0); 
                  wb_slave_ack_s   <= '1';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '0';
               when DTX_addr =>
                  DTX0_s           <= wb_slave_data_i(7  downto 0); 
                  DTX1_s           <= wb_slave_data_i(15 downto 8); 
                  DTX2_s           <= wb_slave_data_i(23 downto 16); 
                  DTX3_s           <= wb_slave_data_i(31 downto 24); 
                  load_TX_s        <= '1';
                  wb_slave_ack_s   <= '1';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '0';
               when others =>
                  wb_slave_ack_s   <= '0';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '1';
               end case;
            when others =>
               case wb_slave_addr_i is
               when STA_addr => 
                  wb_slave_data_o(15 downto 0)  <= STA_i; 
                  wb_slave_data_o(31 downto 16) <= (others => '0');
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when I2C_MASK_addr => 
                  wb_slave_data_o(6 downto 0)   <= I2C_MASK_s(6 downto 0); 
                  wb_slave_data_o(31 downto 7)  <= (others => '0');
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when CTR0_addr => 
                  wb_slave_data_o(14  downto 0)  <= CTR0_s;
                  wb_slave_data_o(31 downto 15) <= (others => '0');
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when CTR1_addr => 
                  wb_slave_data_o(15 downto 0)  <= CTR1_s; 
                  wb_slave_data_o(31 downto 16) <= (others => '0');
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when DRXA_addr => 
                  wb_slave_data_o(7  downto 0)  <= DRX0_i; 
                  wb_slave_data_o(15 downto 8)  <= DRX1_i; 
                  wb_slave_data_o(23 downto 16) <= DRX2_i; 
                  wb_slave_data_o(31 downto 24) <= DRX3_i; 
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when DRXB_addr => 
                  wb_slave_data_o(7  downto 0)  <= DRX4_i; 
                  wb_slave_data_o(15 downto 8)  <= DRX5_i; 
                  wb_slave_data_o(31 downto 16) <= (others => '0');
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when DTX_addr => 
                  wb_slave_data_o(7  downto 0)  <= DTX0_s; 
                  wb_slave_data_o(15 downto 8)  <= DTX1_s; 
                  wb_slave_data_o(23 downto 16) <= DTX2_s; 
                  wb_slave_data_o(31 downto 24) <= DTX3_s;
                  wb_slave_ack_s                <= '1';
                  wb_slave_rty_s                <= '0';
                  wb_slave_err_s                <= '0';
               when others  =>
                  wb_slave_ack_s   <= '0';
                  wb_slave_rty_s   <= '0';
                  wb_slave_err_s   <= '1';
               end case;
            end case;
            end if;
         elsif load_TX_s = '1' then
            load_TX_s      <= '0';
         else
         end if;
      end if;
end process;

end Behavioral;

