--
--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 
--
--   To use any of the example code shown below, uncomment the lines and modify as necessary
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;

package m25p32_pkg is

   ----------------------------------------
   -- ConTRol 0 Register
   ----------------------------------------
   -- 16 bits
   -- Wishbone access: Read-only 
   ----------------------------------------
   --  BIT        NAME         DESCRIPTION
   --   0          OPA          OPeration Attended 
   --   1          OPF          OPeration Finished 
   --   7-2        x            Reserved   
   ----------------------------------------
   type r_CTR0 is
      record
         OPA        : STD_LOGIC;
         OPF        : STD_LOGIC;
         x          : STD_LOGIC_VECTOR(7  downto 2);
      end record;

   ----------------------------------------                                               
   --  Flash Memory Operation Handler Register                                            
   ----------------------------------------                                               
   --  32 bits
   --  Wishbone access: Write-read                                                        
   ----------------------------------------                                               
   --  BIT        NAME      DESCRIPTION 
   --   1-0        OP        OPeration
   --   2          x         Reserved
   --   3          OPP       OPeration Pending
   --   15-4       y         Reserved
   --   23-16      PG        PaGe number
   --   29-24      SCT       SeCTor number
   --   31-30      z         Reserved                                                  
   ----------------------------------------
   --  List of operations:
   --  0b00        x            Reserved  
   --  0b01        ERS           ERase sector. Sets everything to FF
   --  0b10        RDP           ReaD Page. Not implemented by now.
   --  0b11        WRP           WRite Page
   ----------------------------------------
   type t_operations is (x, ERS, RDP, WRP);   --It will be 2 bits

   type r_FMOH is
      record
         OP  : t_operations;
         x   : STD_LOGIC;
         OPP : STD_LOGIC;
         y   : STD_LOGIC_VECTOR(15 downto  4);
         PG  : STD_LOGIC_VECTOR(23 downto 16);
         SCT : STD_LOGIC_VECTOR(29 downto 24);
         z   : STD_LOGIC_VECTOR(31 downto 30);
      end record;

   ------------------------------------------------------------------------------
   --  m25p32 m25p32 m25p32 m25p32 m25p32 SPI Instruction set
   ------------------------------------------------------------------------------
   --  INSTRUCTION        DESCRIPTION                ADDRESS         DATA
   --                                                 BYTES          BYTES
   --    WREN              WRite ENable                 0             0
   --    WRDI              WRite DIsable                0             0
   --    RDSR              ReaD Status Register         0             1
   --    WRSR              WRite Status Register        0             1
   --    READ              READ data bytes              3             4
   --    FAST_READ         FAST READ data bytes         3             4
   --    PP                Page Program                 3             256
   --    SE                Sector Erase                 3             0
   --    BE                Bulk Erase                   0             0
   -------------------------------------------------------------------------------
   constant c_WREN_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"06";
   constant c_WRDI_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"04";
   constant c_RDSR_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"05";
   constant c_WRSR_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"01";
   constant c_READ_inst      : STD_LOGIC_VECTOR(7  downto 0) := X"03";
   constant c_FAST_READ_inst : STD_LOGIC_VECTOR(7  downto 0) := X"0B";
   constant c_PP_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"02";
   constant c_SE_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"D8";
   constant c_BE_inst        : STD_LOGIC_VECTOR(7  downto 0) := X"C7";

   constant c_CTR0_addr      : STD_LOGIC_VECTOR (3  downto 0) := X"0"; 
   constant c_FMOH_addr      : STD_LOGIC_VECTOR (3  downto 0) := X"1";

   constant c_CTR0_default   : r_CTR0 := (OPA => '0',
                                          OPF => '0',
                                          x   => (others => '0'));
   constant c_FMOH_default   : r_FMOH := (OP  =>  t_operations'val(0),
                                          x   => '0',
                                          OPP => '0',
                                          y   => (others => '0'),
                                          PG  => (others => '0'),
                                          SCT => (others => '0'),
                                          z   => (others => '0'));

   function f_STD_LOGIC_VECTOR  (signal r_register : in r_CTR0) return STD_LOGIC_VECTOR;
   function f_STD_LOGIC_VECTOR  (signal r_register : in r_FMOH) return STD_LOGIC_VECTOR;
   function f_CTR0  (signal r_register : in STD_LOGIC_VECTOR(7  downto 0)) return r_CTR0;
   function f_FMOH  (signal r_register : in STD_LOGIC_VECTOR(31 downto 0)) return r_FMOH;



-- procedure <procedure_name> (<type_declaration> <constant_name>	: in <type_declaration>);
--

end m25p32_pkg;

package body m25p32_pkg is

   -----------------------------------------------------------------------------
   --! @brief CTR0 record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_CTR0 record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (signal r_register : in r_CTR0) return STD_LOGIC_VECTOR is
   begin
      return (r_register.x & r_register.OPA & r_register.OPF); 
   end f_STD_LOGIC_VECTOR;

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_CTR0 record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_CTR0  (signal r_register : in STD_LOGIC_VECTOR(7  downto 0)) return r_CTR0 is
      variable  v_CTR0 : r_CTR0;
   begin
      v_CTR0.OPA := r_register(0);
      v_CTR0.OPF := r_register(1);
      v_CTR0.x   := r_register(7 downto 2);
      return v_CTR0; 
   end f_CTR0;

   -----------------------------------------------------------------------------
   --! @brief FMOH record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_FMOH record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (signal r_register : in r_FMOH) return STD_LOGIC_VECTOR is
   begin
      return (r_register.z & r_register.SCT & r_register.PG & r_register.y & r_register.OPP
      & r_register.x & std_logic_vector(to_unsigned(t_operations'pos(r_register.OP),2)) );
   end f_STD_LOGIC_VECTOR; 

   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_CTR0 record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_FMOH(signal r_register : in STD_LOGIC_VECTOR(31  downto 0)) return r_FMOH is
      variable  v_FMOH : r_FMOH;
   begin
      v_FMOH.OP  := t_operations'val(to_integer(unsigned(r_register(t_operations'pos(t_operations'high) downto 0))));
      v_FMOH.x   := r_register(2);
      v_FMOH.OPP := r_register(3);
      v_FMOH.y   := r_register(15 downto  4);
      v_FMOH.PG  := r_register(23 downto 16);
      v_FMOH.SCT := r_register(29 downto 24);
      v_FMOH.z   := r_register(31 downto 30);
      return v_FMOH; 
   end f_FMOH;

end m25p32_pkg;
