----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    00:47:34 10/26/2011 
-- Design Name:    i2c bit recognition fsm
-- Module Name:    i2c_bit - Behavioral 
-- Project Name:   CTDAH
-- Target Devices: Spartan 6
-- Tool versions:  
-- Description:    This is the fsm for the recognition of a bit
--
-- Dependencies:   The inputs should be debounced: i2c_debouncer.vhd
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity i2c_bit is
   generic(
      g_PRESCALER : in STD_LOGIC_VECTOR (15 downto 0) := X"0032"
   );
   port (
      rst_i     : in   STD_LOGIC;
      wb_clk_i  : in   STD_LOGIC;

      presc_e   : in   STD_LOGIC;
      presc     : in   STD_LOGIC_VECTOR (15 downto 0);

      done      : out  STD_LOGIC;
      sda_i     : in   STD_LOGIC;
      scl_i     : in   STD_LOGIC;
      start_o   : out  STD_LOGIC;
      pause_o   : out  STD_LOGIC;
      rcved_o   : out  STD_LOGIC
   );
end i2c_bit;

architecture Behavioral of i2c_bit is

type bit_fsm is (S0_IDLE, S1_HIGH_TMP, S2_LOW_TMP, S3_START_TMP, S4_PAUSE_TMP, Q4_ERROR);

constant c_MAX_GLITCH_DELAY :  UNSIGNED (3 downto 0) := "0110";
constant c_GLITCH_MASK      : STD_LOGIC_VECTOR (5 downto 0) := "000111";      -- Three delay stages out of six

component counter_16
   port (
      clk    : in  STD_LOGIC;
      ce     : in  STD_LOGIC;
      sclr   : in  STD_LOGIC;
      sset   : in  STD_LOGIC;
      q      : out STD_LOGIC_VECTOR(15 downto 0)
   );
end component;

component i2c_debouncer
   generic (
      g_LENGTH    : UNSIGNED(3 downto 0) := c_MAX_GLITCH_DELAY;
		      g_outputONrise : BOOLEAN := TRUE
      );
   port (
      rst         : IN   std_logic;
      clk         : IN   std_logic;                                
      input       : IN   std_logic;
      output      : OUT  std_logic;
      glitch_mask : IN  std_logic_vector(to_integer(g_LENGTH)-1 downto 0)
      );
end component;

component gc_ff
  port(
        Q   : out STD_LOGIC;
        C   : in  STD_LOGIC;
        CLR : in  STD_LOGIC;
        D   : in  STD_LOGIC
      );
end component;

signal sda_s   :  STD_LOGIC;
signal scl_s   :  STD_LOGIC;
signal scl_saux : STD_LOGIC;

signal i2c_bit_fsm    :   bit_fsm;

signal scl_s_d0   : STD_LOGIC;
signal sda_s_d0   : STD_LOGIC;
signal last_scl_s : STD_LOGIC;
signal last_sda_s : STD_LOGIC;

signal scl_rising  : STD_LOGIC;
signal scl_falling : STD_LOGIC;

signal start_s  : STD_LOGIC;
signal pause_s  : STD_LOGIC;
signal rcved_s  : STD_LOGIC;

signal presc_count   : STD_LOGIC_VECTOR (15 downto 0);
signal prescaler     : STD_LOGIC_VECTOR(15 downto 0);
signal presc_ce_s    : STD_LOGIC;
signal sclr_s        : STD_LOGIC;
signal sclr_s_d0     : STD_LOGIC;
signal sclr          : STD_LOGIC;
signal sset_s        : STD_LOGIC;
signal q             : STD_LOGIC;  

begin

--start_o    <= start_s;
--pause_o    <= pause_s;
--rcved_o    <= rcved_s;

--done       <= scl_falling;

   inst_counter_16: counter_16
   port map (
      clk    => wb_clk_i,
      ce     => presc_ce_s,
      sclr   => sclr,
      sset   => sset_s,
      q      => presc_count
   );

   debouncer_scl_i: i2c_debouncer
   generic map(
      g_outputONrise => TRUE
   )
   port map(
      rst          => rst_i,
      clk          => wb_clk_i,
      input        => scl_i,
      output       => scl_s,
      glitch_mask  => c_GLITCH_MASK
   );

   ff1_scl : gc_ff
   port map(
      Q => scl_s_d0,
      C => wb_clk_i,
      CLR => rst_i,
      D => scl_s
   );

   ff2_scl : gc_ff
   port map(
      Q => last_scl_s,
      C => wb_clk_i,
      CLR => rst_i,
      D => scl_s_d0
   );


   debouncer_sda_i: i2c_debouncer
   generic map(
      g_outputONrise => TRUE
   )
   port map(
      rst          => rst_i,
      clk          => wb_clk_i,
      input        => sda_i,
      output       => sda_s,
      glitch_mask  => c_GLITCH_MASK
   );

   ff1_sda : gc_ff
   port map(
      Q => sda_s_d0,
      C => wb_clk_i,
      CLR => rst_i,
      D => sda_s
   );

   ff2_sda : gc_ff
   port map(
      Q => last_sda_s,
      C => wb_clk_i,
      CLR => rst_i,
      D => sda_s_d0
   );

process (wb_clk_i)

   procedure reset_proc is
   begin
      i2c_bit_fsm    <= S0_IDLE;
      start_s       <= '0';
      pause_s       <= '0';
      rcved_s       <= '1';
                -- Added to solve problems in upper levels
      start_o       <= '0';
      pause_o       <= '0';
      rcved_o       <= '1';
		--
      scl_rising    <= '0';
      scl_falling   <= '0';
      done          <= '0';
      sset_s        <= '0';
      sclr_s        <= '0';
   end reset_proc;

   procedure check_fsm is
   begin
     case scl_s is
       when  '1' =>
         case i2c_bit_fsm is
           when S0_IDLE      =>
             case sda_s is 
               when '1'    =>
                 i2c_bit_fsm   <= S1_HIGH_TMP;
               --               start_s   <= '0';
               --               pause_s   <= '0';
               --               rcved_s   <= '1';
                 start_o   <= '0';
                 pause_o   <= '0';
                 rcved_o   <= '1';
               when others =>
                 i2c_bit_fsm   <= S2_LOW_TMP;
               --               start_s   <= '0';
               --               pause_s   <= '0';
               --               rcved_s   <= '0';
                 start_o   <= '0';
                 pause_o   <= '0';
                 rcved_o   <= '0';
             end case; 
           when S1_HIGH_TMP  =>
             case sda_s is
               when '1'    =>
                 i2c_bit_fsm   <= S1_HIGH_TMP;
               when others =>
                 i2c_bit_fsm   <= S3_START_TMP;
               --               start_s       <= '1';
                 start_o       <= '1';
             end case;
           when S2_LOW_TMP   =>
             case sda_s is
               when '0'    =>
                 i2c_bit_fsm   <= S2_LOW_TMP;
               when others =>
                 i2c_bit_fsm   <= S4_PAUSE_TMP;
               --               pause_s       <= '1';
                 pause_o       <= '1';
             end case;
           when S3_START_TMP =>
             case sda_s is
               when '0'    =>
                 i2c_bit_fsm   <= S3_START_TMP;
               when others =>
                 i2c_bit_fsm   <= Q4_ERROR;
             end case;
           when S4_PAUSE_TMP =>
             case sda_s is
               when '1'    =>
                 i2c_bit_fsm   <= S4_PAUSE_TMP;
               when others =>
                 i2c_bit_fsm   <= Q4_ERROR;
             end case;
           when Q4_ERROR =>
                                --            start_s     <= '1';
                                --            pause_s     <= '1';
             start_o     <= '1';
             pause_o     <= '1';
         end case;
       when others =>
     end case;
   end check_fsm;
 
 begin
 
--  TODO: Solve these lines to make it synthesizable
--    if presc_count = prescaler then
--       sclr_s    <= '1';
----		 sclr_o    <= '1';
--    else
--       sclr_s    <= '0';
----		 sclr_o    <= '0';
--    end if;
--
--   if presc_e = '1' then
--      prescaler    <= presc;
--   else
--   end if;

   if rising_edge(wb_clk_i) then
      if rst_i = '1' then
         reset_proc;
         prescaler     <= g_PRESCALER;

--      elsif (rising_edge(scl_s) or falling_edge(scl_s)) then
      elsif (scl_s_d0 xor scl_s) = '1' then
         if scl_s = '1' then
           scl_rising  <= '1';
         else
           scl_falling <= '1';
           done        <= '1';
         end if;   
      check_fsm;

      elsif scl_falling = '1' then
         reset_proc;

      elsif scl_rising = '1' then
         scl_rising   <= '0';
         check_fsm;
      else
         check_fsm;
      end if;

   else
--      if rst_i = '1' then
--         reset_proc;
--         prescaler     <= g_PRESCALER;
--      else
--      end if;
  end if;

end process;


end Behavioral;

