----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    17:50:12 11/25/2011 
-- Design Name:    i2c_wb_access_ctrl
-- Module Name:    i2c_wb_access_ctrl - Behavioral 
-- Project Name:   i2c to wishbone access controller
-- Target Devices: Spartan 6
-- Tool versions: 
-- Description:    This is the moduel that controls the access to the wishbone modules
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity i2c_wb_access_ctrl is
   generic(
      g_WISHBONE_MODULES: NATURAL := 4;
      g_WISHBONE_MEMWIDTH:   NATURAL := 16
          );
   port (
      rst           : in   STD_LOGIC;
      wb_clk        : in   STD_LOGIC;
      wb_stb_o      : out  STD_LOGIC;
      wb_cyc_o      : out  STD_LOGIC;
      wb_sel_o      : out  STD_LOGIC;
      wb_we_o       : out  STD_LOGIC;
      wb_data_i     : in   STD_LOGIC_VECTOR (31 downto 0);      
      wb_data_o     : out  STD_LOGIC_VECTOR (31 downto 0);
      wb_addr_o     : out  STD_LOGIC_VECTOR (3  downto 0);
      wb_ack_i      : in   STD_LOGIC;
      wb_rty_i      : in   STD_LOGIC;
      wb_err_i      : in   STD_LOGIC;
      -- Interruptions
      ind_wb_addr_i : in   STD_LOGIC;
      inst_rd_i     : in   STD_LOGIC;
      inst_wr       : in   STD_LOGIC;
      -- Here we add all the wishbone ports needed
      wb_cyc_i      : in   STD_LOGIC_VECTOR (g_WISHBONE_MODULES - 1 downto 0);
      wb_sel_i      : in   STD_LOGIC_VECTOR (g_WISHBONE_MODULES - 1 downto 0);
      wb_we_i       : in   STD_LOGIC_VECTOR (g_WISHBONE_MODULES - 1 downto 0);
      wb_data_o     : out  STD_LOGIC_VECTOR (32*g_WISHBONE_MODULES -1 downto 0);      
      wb_data_i     : in   STD_LOGIC_VECTOR (32*g_WISHBONE_MODULES -1 downto 0);
      wb_addr_i     : in   STD_LOGIC_VECTOR (g_WISHBONE_MEMWIDTH*g_WISHBONE_MODULES - 1 downto 0);
      wb_ack_o      : out  STD_LOGIC_VECTOR (g_WISHBONE_MEMWIDTH*g_WISHBONE_MODULES - 1 downto 0);
      wb_rty_o      : out  STD_LOGIC_VECTOR (g_WISHBONE_MEMWIDTH*g_WISHBONE_MODULES - 1 downto 0);
      wb_err_o      : out  STD_LOGIC_VECTOR (g_WISHBONE_MEMWIDTH*g_WISHBONE_MODULES - 1 downto 0)
   );
end CTDAH_top;

architecture Behavioral of CTDAH_top is

constant CTDAH_control_addr    : STD_LOGIC_VECTOR (15 downto 0) := X"0100"
constant i2c_slave_addr        : STD_LOGIC_VECTOR (15 downto 0) := X"0200";
constant trigger_addr          : STD_LOGIC_VECTOR (15 downto 0) := X"0300";
constant EEPROM_manager_addr   : STD_LOGIC_VECTOR (15 downto 0) := X"0400";
constant wr_core_addr          : STD_LOGIC_VECTOR (15 downto 0) := X"0500";
constant EEPROM_memory         : STD_LOGIC_VECTOR (15 downto 0) := X"1000";

-------------------------------------------------------------------------------
-- [ 15  | 14 | 13 | 12 | 11 | 10 | 9 | 8 |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  ]
-- [                 RESERVED             |  X  | EN5 | EN4 | EN3 | EN2 | EN1 | EN0 | GEN ]
-------------------------------------------------------------------------------
--  GEN  this is the general enable
--  ENi  enable access to the given wishbone module
-------------------------------------------------------------------------------  
signal CTRL0: STD_LOGIC_VECTOR (15 downto 0);

-------------------------------------------------------------------------------
-- [ 15  | 14 |  13 |  12 |  11 |  10 |  9  |  8  |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  ]
-- [  x  |  x | RD5 | WR5 | RD4 | WR4 | RD3 | WR3 | RD2 | WR2 | RD1 | WR1 | RD0 | WR0 | GRD | GWR ]
-------------------------------------------------------------------------------
-- 
-------------------------------------------------------------------------------
signal CTRL1: STD_LOGIC_VECTOR (15 downto 0);

begin

  process(wb_clk)
    variable caseCond0 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(3)  & CTR1(2);
    variable caseCond1 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(5)  & CTR1(4);
    variable caseCond2 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(7)  & CTR1(6);
    variable caseCond3 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(9)  & CTR1(8);
    variable caseCond4 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(11) & CTR1(10);
    variable caseCond5 : STD_LOGIC_VECTOR (1 downto 0) := CTRL1(13) & CTR1(12);
  begin

    if rising_edge(wb_clk)then
      if ind_wb_addr_i = '1' then

       -- We do some prefetch depending upon permisssions
        case wb_data_o(15 downto 0) is
          when CTDAH_control_addr  =>
            if CTRL0(0) and CTRL0 (1) then
              case caseCond0 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
            else
            end if;
          when i2c_slave_addr      =>
            if CTRL0(0) and CTRL0 (2) then
               case caseCond1 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
           else
            end if;
          when trigger_addr        =>
            if CTRL0(0) and CTRL0 (3) then
               case caseCond2 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
           else
            end if;
          when EEPROM_manager_addr =>
            if CTRL0(0) and CTRL0 (4) then
               case caseCond3 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
           else
            end if;
          when wr_core_addr        =>
            if CTRL0(0) and CTRL0 (5) then
               case caseCond4 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
           else
            end if;
          when EEPROM_memory       =>
            if CTRL0(0) and CTRL0 (6) then
               case caseCond5 is
                when "10"=>
                  -- Read permissions
                when "01"=>
                  -- Write permissions
                when "11"=>
                  -- Read & write permissions
                when others =>
              end case;
           else
            end if;
          when others =>
        end case;

      else
      end if;
    else
    end if;

  end process;

end Behavioral;


