----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    17:34:27 06/14/2012 
-- Design Name:     
-- Module Name:    spi_master_core - Behavioral 
-- Project Name:   CTDAH/SVEC
-- Target Devices: Spartan 6
-- Tool versions:  Xilinx ISE 13.4
-- Description:    This is a spi_master with programable field length and progra
--                 mable setup and hold chip select times.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.spi_master_pkg.ALL;
use work.ctdah_pkg.ALL;

entity spi_master_core is
   generic(
   g_INST_LENGTH  : NATURAL := c_INST_LENGTH;
   g_ADDR_LENGTH  : NATURAL := c_ADDR_LENGTH;
   g_DATA_LENGTH  : NATURAL := c_DATA_LENGTH;
   g_CLK_I_PERIOD : NATURAL := c_CLK_I_PERIOD);
   port(
             rst_i         : in  STD_LOGIC;
             clk_i         : in  STD_LOGIC;
   
             inst_i        : in  STD_LOGIC_VECTOR (8*g_INST_LENGTH - 1 downto 0);
             addr_i        : in  STD_LOGIC_VECTOR (8*g_ADDR_LENGTH - 1 downto 0);
             data_i        : in  STD_LOGIC_VECTOR (8*g_DATA_LENGTH - 1 downto 0);
   

             SPI0_i        : in  STD_LOGIC_VECTOR (31 downto 0);
             SPI1_i        : in  STD_LOGIC_VECTOR (31 downto 0);

             spi_mosi_o    : out STD_LOGIC;
             spi_miso_i    : in  STD_LOGIC;
             spi_clk_o     : out STD_LOGIC;
             spi_cs_n_o    : out STD_LOGIC
          );
   end spi_master_core;

architecture Behavioral of spi_master_core is

signal s_inst_reg_o    : STD_LOGIC_VECTOR (7 downto 0);
signal s_addr_reg_o    : STD_LOGIC_VECTOR (7 downto 0);
signal s_data_reg_o    : STD_LOGIC_VECTOR (7 downto 0);

signal s_SPI0          : r_SPI0;
signal s_SPI1          : r_SPI1;

type SPI_fsm is (S0_IDLE,
                 S1_SETUP_SPI,
                 S2_INST,
                 S3_ADDR,
                 S4_DATA,
                 S5_HOLD_SPI
              );

type r_STATUS is
   record
      fsm          : SPI_fsm;
      inst_sent    : STD_LOGIC;
      addr_sent    : STD_LOGIC;
      data_sent    : STD_LOGIC;
   end record;

signal   s_STATUS         : r_STATUS;
constant c_STATUS_DEFAULT : r_STATUS := (fsm       => S0_IDLE,
                                         inst_sent => '0',
                                         addr_sent => '0',
                                         data_sent => '0');

constant c_CS_SU_TICKS          : NATURAL := c_CS_SU_SPI/c_CLK_I_PERIOD + 1;
constant c_CS_HOLD_TICKS        : NATURAL := c_CS_HOLD_SPI/c_CLK_I_PERIOD + 1;


signal s_clk_div_oe_n         : STD_LOGIC;

signal s_timing_counter_en           : STD_LOGIC;
signal s_timing_counter_manual_rst_i : STD_LOGIC;
signal s_timing_counter_rst_i        : STD_LOGIC;
signal s_timing_counter_cnt          : STD_LOGIC_VECTOR(c_COUNTER_DATA_WIDTH - 1 downto 0);

signal s_spi_counter_en           : STD_LOGIC;
signal s_spi_counter_manual_rst_i : STD_LOGIC;
signal s_spi_counter_rst_i        : STD_LOGIC;
signal s_spi_counter_cnt          : STD_LOGIC_VECTOR (c_COUNTER_DATA_WIDTH - 1 downto 0);

signal s_spi_clk                  : STD_LOGIC;
signal s_spi_clk_divider_i        : STD_LOGIC_VECTOR (15  downto 12);
begin


   spi_clk_o     <= s_spi_clk;
   s_spi_clk_divider_i <= std_logic_vector(s_SPI1.CLK_DIV);

   s_SPI0   <= f_SPI0(SPI0_i);
   s_SPI1   <= f_SPI1(SPI1_i);

   s_spi_counter_rst_i    <= s_spi_counter_manual_rst_i    or rst_i;
   s_timing_counter_rst_i <= s_timing_counter_manual_rst_i or rst_i;

   inst_fifo_i2c: FIFO_dispatcher
   generic map(
      g_data_width => 8, g_dispatcher_depth => c_INST_LENGTH)
   port map(
      reg_i   => inst_i,
      clk     => clk_i,
      load    => s_SPI1.PUSH_INST,
      flush   => rst_i,
      oen_i   => s_SPI1.PULL_INST,
      reg_o   => s_inst_reg_o 
   );

   addr_fifo_i2c: FIFO_dispatcher
   generic map(
      g_data_width => 8, g_dispatcher_depth => c_ADDR_LENGTH)
   port map(
      reg_i   => addr_i,
      clk     => clk_i,
      load    => s_SPI1.PUSH_ADDR,
      flush   => rst_i,
      oen_i   => s_SPI1.PULL_ADDR,
      reg_o   => s_addr_reg_o
   );

   data_fifo_i2c: FIFO_dispatcher
   generic map(
      g_data_width => 8, g_dispatcher_depth => c_DATA_LENGTH)
   port map(
      reg_i   => data_i,
      clk     => clk_i,
      load    => s_SPI1.PUSH_ADDR,
      flush   => rst_i,
      oen_i   => s_SPI1.PULL_ADDR,
      reg_o   => s_data_reg_o
   );

   spi_cclk: gc_clk_divider
   generic map( g_clk_division_logSize => 4)
   port map(
              clk_i      => clk_i, 
              rst_i      => rst_i, 
              oe_n_i     => s_clk_div_oe_n, 
              clk_o      => s_spi_clk, 
              divider_i  => s_spi_clk_divider_i 
           );

   timing_counter: gc_counter
   generic map( g_data_width => c_COUNTER_DATA_WIDTH)
   port map(
              clk_i  => clk_i,
              rst_i  => s_timing_counter_rst_i,
              en_i   => s_timing_counter_en,
              cnt_o  => s_timing_counter_cnt
              );

   spi_bit_counter: gc_counter
   generic map( g_data_width => c_COUNTER_DATA_WIDTH)
   port map(
              clk_i  => s_spi_clk,
              rst_i  => s_spi_counter_rst_i,
              en_i   => s_spi_counter_en,
              cnt_o  => s_spi_counter_cnt
              );


   p_spi_serializer_out : process(clk_i)
   begin
      if rising_edge(clk_i) then
         if rst_i = '1' then
               spi_mosi_o  <= '0';
               spi_cs_n_o  <= '1';
         else
            case s_STATUS.fsm is
               when S0_IDLE        =>
                  spi_mosi_o     <= '0';
                  s_clk_div_oe_n <= '1';
                  spi_cs_n_o     <= '1';
               when S1_SETUP_SPI   =>
                  spi_mosi_o     <= '0';
                  s_clk_div_oe_n <= '1';
                  spi_cs_n_o     <= '0';
               when S2_INST        =>
                  if unsigned(s_timing_counter_cnt) = 1 AND s_spi_clk = '0' then
                     spi_mosi_o  <= s_inst_reg_o(c_INST_LENGTH*8 - to_integer(unsigned(s_spi_counter_cnt)));
                  else
                  end if;
               when S3_ADDR        =>
                  if unsigned(s_timing_counter_cnt) = 1 AND s_spi_clk = '0' then
                     spi_mosi_o  <= s_inst_reg_o(c_ADDR_LENGTH*8 - to_integer(unsigned(s_spi_counter_cnt)));
                  else
                  end if;
               when S4_DATA        =>
                  if unsigned(s_timing_counter_cnt) = 1 AND s_spi_clk = '0' then
                     spi_mosi_o  <= s_inst_reg_o(c_DATA_LENGTH*8 - to_integer(unsigned(s_spi_counter_cnt)));
                  else
                  end if;
               when S5_HOLD_SPI    =>
                  spi_mosi_o     <= '0';
                  s_clk_div_oe_n <= '1';
                  spi_cs_n_o     <= '0';
               when others         =>

            end case;
         end if;
      else
      end if;
   end process;


   p_spi_serializer_fsm : process(s_spi_clk)
   begin
      if rising_edge(s_spi_clk) then
         if rst_i = '1' then
            s_STATUS   <= c_STATUS_DEFAULT;
         else
            if s_SPI1.SEND_OP = '1' then
               case s_STATUS.fsm is
                  when S0_IDLE =>
                     
                     if s_SPI1.SEND_INST = '1' then 
                        s_STATUS.fsm                  <= S1_SETUP_SPI;
                        s_timing_counter_en           <= '1';
                        s_timing_counter_manual_rst_i <= '1';
                     else
                        s_STATUS         <= c_STATUS_DEFAULT;
                        s_timing_counter_manual_rst_i  <= '0';
                     end if;
                  when S1_SETUP_SPI =>
                     if unsigned(s_timing_counter_cnt) < c_CS_SU_TICKS then
                        if s_SPI1.SEND_INST = '1' then
                           s_STATUS.fsm                  <= S2_INST;
                           s_timing_counter_manual_rst_i <= '1';
                        else
                           s_STATUS.fsm     <= S5_HOLD_SPI;
                           s_timing_counter_manual_rst_i  <= '0';
                        end if;
                     else
                     end if;
                  when S2_INST =>
                     if unsigned(s_spi_counter_cnt) = (s_SPI0.BINST*8) then
                        s_spi_counter_manual_rst_i <= '1';
                        if s_SPI1.SEND_ADDR = '1' then 
                           s_STATUS.fsm                  <= S3_ADDR;
                           s_timing_counter_manual_rst_i <= '1';
                        elsif s_SPI1.SEND_DATA = '1' then 
                           s_STATUS.fsm     <= S4_DATA;
                           s_timing_counter_manual_rst_i  <= '0';
                        else
                           s_STATUS.fsm     <= S5_HOLD_SPI;
                           s_timing_counter_manual_rst_i  <= '0';
                        end if;
                     else
                        s_spi_counter_manual_rst_i  <= '0';
                     end if;
                  when S3_ADDR =>
                     if unsigned(s_spi_counter_cnt) =  (s_SPI0.BADDR*8) then
                        s_timing_counter_manual_rst_i <= '1';
                        if s_SPI1.SEND_DATA = '1' then 
                           s_STATUS.fsm     <= S4_DATA;
                        else
                           s_STATUS.fsm     <= S5_HOLD_SPI;
                        end if;
                     else
                     end if;
                  when S4_DATA =>
                     if unsigned(s_spi_counter_cnt) =  (s_SPI0.BDATA*8) then
                        s_timing_counter_manual_rst_i <= '1';
                     else
                        s_timing_counter_manual_rst_i  <= '0';
                     end if;
                  when S5_HOLD_SPI =>
                     if unsigned(s_timing_counter_cnt) < c_CS_HOLD_TICKS then
                        s_timing_counter_manual_rst_i  <= '0';
                     else
                        s_STATUS               <= c_STATUS_DEFAULT;
                        s_timing_counter_en    <= '0';
                        s_timing_counter_manual_rst_i <= '1';
                     end if;
                  when others  =>
               end case;
            else
            end if;
         end if;
      else
      end if;
   end process;


end Behavioral;
