--
--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 
--
--   To use any of the example code shown below, uncomment the lines and modify as necessary
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;

package spi_master_pkg is

   ----------------------------------------
   -- SPI0 register
   ----------------------------------------
   -- 32 bits
   -- Wishbone access: Write-read
   ----------------------------------------
   --  BIT        NAME         DESCRIPTION
   --    7-0       x             Reserved
   --   15-8       BDATA         Bytes of DATA to be sent        
   --   23-16      BADDR         Bytes of ADDRess to be sent  
   --   31-24      BINST         Bytes of INSTruction to be sent  
   ----------------------------------------
   type r_SPI0 is
      record
         x        : STD_LOGIC_VECTOR (7 downto 0);
         BDATA    : UNSIGNED (15 downto  8); 
         BADDR    : UNSIGNED (23 downto 16); 
         BINST    : UNSIGNED (31 downto 24); 
      end record;

   ----------------------------------------
   -- SPI1 register
   ----------------------------------------
   -- 32 bits
   -- Wishbone access: Write-read
   ----------------------------------------
   --  BIT        NAME         DESCRIPTION
   --    0         PUSH_DATA     PUSH DATA bytes into internal SPI
   --                             core memory 
   --    1         PUSH_ADDR     PUSH ADDRess bytes into internal SPI
   --                             core memory  
   --    2         PUSH_INST     PUSH INSTruction bytes into internal
   --                             SPI core memory 
   --    3         PULL_DATA     PULL one DATA byte from SPI data FIFO
   --    4         PULL_ADDR     PULL one ADDRess byte from SPI address FIFO
   --    5         PULL_INST     PULL one INSTruction byte from SPI
   --                             instruction FIFO
   --    6         SEND_DATA     DATA bytes will be sent in a write operation 
   --    7         SEND_ADDR     ADDR bytes will be sent in a write operation
   --    8         SEND_INST     INST bytes will be sent in a write operation
   --    9         SEND_OP       perform a SEND OPeration
   --   11-10      x             Reserved
   --   15-12      CLK_DIV       CLocK DIVider
   --   31-16      y             Reserved
   ----------------------------------------
   type r_SPI1 is
      record
         PUSH_DATA    : STD_LOGIC; 
         PUSH_ADDR    : STD_LOGIC; 
         PUSH_INST    : STD_LOGIC;
         PULL_DATA    : STD_LOGIC; 
         PULL_ADDR    : STD_LOGIC; 
         PULL_INST    : STD_LOGIC;
         SEND_DATA    : STD_LOGIC;
         SEND_ADDR    : STD_LOGIC;
         SEND_INST    : STD_LOGIC;
         SEND_OP      : STD_LOGIC;
         x            : STD_LOGIC_VECTOR (11 downto 10);
         CLK_DIV      : UNSIGNED (15 downto 12);
         y            : STD_LOGIC_VECTOR (31 downto 16);
      end record;

   constant c_INST_LENGTH          : NATURAL := 1; 
   constant c_ADDR_LENGTH          : NATURAL := 3;   
   constant c_DATA_LENGTH          : NATURAL := 256;

   constant c_CLK_DIVISION_LOGSIZE : NATURAL := 2;
   constant c_COUNTER_DATA_WIDTH   : NATURAL := 8;

   constant c_CLK_I_PERIOD         : NATURAL := 50;  --! ns
   constant c_CS_SU_SPI            : NATURAL := 20;  --! ns
--   constant c_CS_SU_TICKS          : NATURAL := c_CS_SU_SPI/c_CLK_I_PERIOD + 1;
   constant c_CS_HOLD_SPI          : NATURAL := 100; --! ns
--   constant c_CS_HOLD_TICKS        : NATURAL := c_CS_HOLD_SPI/c_CLK_I_PERIOD + 1;

   constant c_SPI0_addr  : STD_LOGIC_VECTOR (3  downto 0) := X"0"; 
   constant c_SPI1_addr  : STD_LOGIC_VECTOR (3  downto 0) := X"1"; 

   function f_SPI0  (signal r_register : in STD_LOGIC_VECTOR(31 downto 0)) return r_SPI0;
   function f_SPI1  (signal r_register : in STD_LOGIC_VECTOR(31 downto 0)) return r_SPI1;

end spi_master_pkg;

package body spi_master_pkg is

   -----------------------------------------------------------------------------
   --! @brief CTR0 record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_SPI0 record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (signal r_register : in r_SPI0) return STD_LOGIC_VECTOR is
   begin
      return (  std_logic_vector(r_register.BINST)
              & std_logic_vector(r_register.BADDR)
              & std_logic_vector(r_register.BDATA)
              & r_register.x); 
   end f_STD_LOGIC_VECTOR;


   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_SPI0 record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_SPI0  (signal r_register : in STD_LOGIC_VECTOR(31  downto 0)) return r_SPI0 is
      variable  v_SPI0 : r_SPI0;
   begin
      v_SPI0.x       := r_register(7  downto 0); 
      v_SPI0.BDATA   := unsigned(r_register(15 downto  8)); 
      v_SPI0.BADDR   := unsigned(r_register(23 downto 16)); 
      v_SPI0.BINST   := unsigned(r_register(31 downto 24)); 
      return v_SPI0;
   end f_SPI0;


   -----------------------------------------------------------------------------
   --! @brief CTR0 record type translation to STD_LOGIC_VECTOR
   --! @param r_register r_CTR0 record type to be translated
   -----------------------------------------------------------------------------
   function f_STD_LOGIC_VECTOR  (signal r_register : in r_SPI1) return STD_LOGIC_VECTOR is
   begin
      return (  r_register.y & std_logic_vector(r_register.CLK_DIV)
              & r_register.x & r_register.SEND_OP
              & r_register.SEND_INST & r_register.SEND_ADDR
              & r_register.SEND_DATA
              & r_register.PULL_INST & r_register.PULL_ADDR
              & r_register.PULL_DATA
              & r_register.PUSH_INST & r_register.PUSH_ADDR
              & r_register.PUSH_DATA
           ); 
   end f_STD_LOGIC_VECTOR;


   -----------------------------------------------------------------------------
   --! @brief Translation from STD_LOGIC_VECTOR to r_SPI1 record
   --! @param r_register STD_LOGIC_VECTOR to be translated
   -----------------------------------------------------------------------------
   function f_SPI1  (signal r_register : in STD_LOGIC_VECTOR(31  downto 0)) return r_SPI1 is
      variable  v_SPI1 : r_SPI1;
   begin
      v_SPI1.PUSH_DATA   := r_register(0); 
      v_SPI1.PUSH_ADDR   := r_register(1); 
      v_SPI1.PUSH_INST   := r_register(2); 
      v_SPI1.PULL_DATA   := r_register(3); 
      v_SPI1.PULL_ADDR   := r_register(4); 
      v_SPI1.PULL_INST   := r_register(5); 
      v_SPI1.SEND_DATA   := r_register(6); 
      v_SPI1.SEND_ADDR   := r_register(7); 
      v_SPI1.SEND_INST   := r_register(8); 
      v_SPI1.SEND_OP     := r_register(9); 
      v_SPI1.x           := r_register(11 downto 10); 
      v_SPI1.CLK_DIV     := unsigned(r_register(15 downto 12)); 
      v_SPI1.y           := r_register(31 downto 16); 
      return v_SPI1;
   end f_SPI1;



   end spi_master_pkg;
